/***************************************************************************                          
    file:           ksensorsmain.cpp
    description:    ksensors main
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qevent.h>
#include <qapplication.h>

#include <klocalizedstring.h>
#include <kaboutapplicationdialog.h>
#include <kwindowsystem.h>
#include <kx11extras.h>
#include <kaboutdata.h>
#include <kwindowinfo.h>

#include <netwm.h>

#include "general.h"
#include "sensorscfg.h"
#include "ksensorsmain.h"


#include "qlchart.h"


KsensorsMain::KsensorsMain(QObject *parent, const QString &name): QObject(/* parent,name */)
{
    Q_UNUSED(parent);
    Q_UNUSED(name);

    gaugePanelsWindow = nullptr;
    chartPanelsWindow = nullptr;

    sensorsCfg = nullptr;    
    sensorsAlarm = nullptr;

    qmContext = nullptr;
    qaMinimizeRestore = nullptr;

    gaugeIncrWidth = gaugeIncrHeight = chartIncrWidth = chartIncrHeight = 64;
    
    idTimer = 0;
    updateInterval = 2000;
    updateIntervalTimer = 500;

    sensorDevsList= new SensorDevsList ( this );
    sensorDevsList->setMonitorized( true );

    setMeterSizeList();
//    init();
}

KsensorsMain::~KsensorsMain()
{
    saveConfig ( false );

    if ( idTimer )
    {
        killTimer ( idTimer );
        idTimer = 0;
    }

 if(gaugePanelsWindow) delete gaugePanelsWindow;
 if(sensorsCfg) delete sensorsCfg;
 if(sensorsAlarm) delete sensorsAlarm;
 
 qmContext->deleteLater();
 qmContext = nullptr;

 if(sensorDevsList) delete sensorDevsList;
}

/***************************************************************************
    initialize
 ***************************************************************************/

void KsensorsMain::init()
{
    QList<QString> gaugeSizeList;
    QList<QString> chartWidthList;
    QList<QString> chartHeightList;

    
    
    createAlarmControl();
/*  createWidgets();
    createMenu (qmContext, qaMinimizeRestore); 
    if(!getPar ( qsFL1( "nodockopt" ), false ) ) createDockWidgets(); */
// createMenu();

    updateSensorsPanelsDocks( qsFL1( "_init_" ), false );
    createMenu (qmContext, qaMinimizeRestore);
    if(!getPar ( qsFL1( "nodockopt" ), false ) ) createDockWidgets();

    if ( getPar( qsFL1( "charttestopt" ), false ) )
    {
        testChart *wcChart = new testChart();
        wcChart->show();
    }
    
/* warning, it would be better not to have the global timer active without any sensor enabled,
   since the sensor lists are monitored by default, the timer is activated) */
    idTimer = startTimer( updateIntervalTimer );
}

void KsensorsMain::createMenu ( QMenu *&qmMenu, QAction *&qaMinRest )
{
  const KAboutData kadAboutData = KAboutData::applicationData();
  
  qmMenu = new QMenu();

  qmMenu->addAction( QIcon::fromTheme( qsFL1( COMPICON ) ), kadAboutData.displayName() );
  qmMenu->addSeparator();  
  qmMenu->addAction( QIcon::fromTheme( qsFL1( "configure") ), i18n( "&Configure" ), this, SLOT(createConfigWidget() ));
  qmMenu->addAction( QIcon ( findIcon ( "harddisk.png" ) ), i18n( "&Save Configuration" ), this, [=, this]() { this->saveConfig( true ); } );
  qmMenu->addSeparator();
  qmMenu->addAction( QIcon::fromTheme( qsFL1( COMPICON ) ), i18n( "&About ") + kadAboutData.displayName(), this, SLOT(showAbout() ));
  qmMenu->addSeparator();
    
  qaMinRest = qmMenu->addAction( ( gaugePanelsWindow || chartPanelsWindow ) ? i18n("&Close"):i18n("&Restore"), this, SLOT(minimizeRestoreWidget() ));
  qaMinRest->setEnabled( !getPar ( qsFL1( "nodockopt" ), false ) );
  
  qmMenu->addAction( QIcon::fromTheme( qsFL1( "exit" ) ), i18n("&Exit"), this, SLOT(close() ));
}

/***************************************************************************
    update value
 ***************************************************************************/

void KsensorsMain::updateValue( const QString & name )
{
    if ( gaugePanelsWindow )
        gaugePanelsWindow->updateValue ( name );

    if ( chartPanelsWindow )
        chartPanelsWindow->updateValue ( name );
}

void KsensorsMain::saveConfig ( bool enabSync )
{
//    qDebug() << "\033[91m" << "qDebug(): ksensorsMain::saveConfig enabSync" << enabSync <<"\033[0m";  

/* warning , this part needs to be completely revised.
** in general the panels window should directly do all the necessary operations. */
    
    setGroupConf ( qsFL1( "General" ) );
    setEntryConf ( qsFL1( "showWidget" ), ( (bool) gaugePanelsWindow || (bool) gaugePanelsWindow ) );

    if ( gaugePanelsWindow )
    {
        gaugePanelsWindow->saveGeometry();

        KWindowInfo winInfo (gaugePanelsWindow->winId(), NET::WMDesktop);
        if ( winInfo.valid() )
            setEntryConf ( qsFL1( "desktop" ), winInfo.desktop() );
    }

    if ( chartPanelsWindow )
    {
        chartPanelsWindow->saveGeometry();

        KWindowInfo winInfo (chartPanelsWindow->winId(), NET::WMDesktop);
        if ( winInfo.valid() )
            setEntryConf ( qsFL1( "desktop" ), winInfo.desktop() );
    }
    
    if ( enabSync )
        syncConf();
}

/***************************************************************************
    update sensors config
 ***************************************************************************/

void KsensorsMain::updateConfig ( Updates updatesType, struct T_UpdateData &updateData )
{
    int iMaxCommDiv, iMin, iMax;

    Updates updatesSensorsType;

    struct T_UpdateData updateSensorsData;

    updatesSensorsType = updatesType;
    
    updateSensorsData.init();
    updateSensorsData.withChange = updateData.withChange;

    if ( ! ( updatesType & KsensorsMain::Initialize ) )
    {
        if ( !updateData.sensorName.isEmpty() && updateData.sensorName != qsFL1( "General" ) )
            updateSensorsData.sensorName = updateData.sensorName;
    }


    setGroupConf ( qsFL1( "General" ) );
    updateSensorsData.genUpdateInterval = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, Type::AutoDefault ) * 1000;


    updateSensorsConfig ( updatesSensorsType, updateSensorsData );


    updateInterval = updateSensorsData.genUpdateInterval;
    updateIntervalTimer = updateInterval;

    if ( !updateSensorsData.updateIntervalList.isEmpty() )
    {
        maxCommonDivisor ( updateSensorsData.updateIntervalList, iMaxCommDiv, iMin, iMax );
        updateIntervalTimer = iMaxCommDiv;
    }

} // void updateConfig ( Updates updatesType, struct T_UpdateData &updateData )

void KsensorsMain::updateSensorsConfig ( Updates updatesType, struct T_UpdateData &updateData )
{
    QString sensorDevName, sensorName;
    QString qsSensorShowType;
    MeterTypes sensorMeterTypes;

    const QObjectList *chips;

    chips = sensorDevsList->getSensorsChips();

    sensorName = sensorDevName = updateData.sensorName;

    if ( updatesType & KsensorsMain::UpdateSensorsList)
        sensorDevsList->qmmSensorDevsList.clear();

    if( chips )
    {
        for ( QObjectList::ConstIterator it = chips->begin(); it != chips->end(); ++it )
        {
            SensorDev *chip = ( SensorDev * )( *it );

            if( chip )
            {
                if ( sensorDevName.isEmpty() || ( chip->getLongName() == sensorDevName ) )
                {
                    sensorName.clear();

                    if ( updatesType & KsensorsMain::UpdateSensorsList )
                        sensorDevsList->qmmSensorDevsList.insert ( QString::number ( chip->getClass() ) + qsFL1( "_" ) + chip->getLongName(), chip) ;

                    if ( updatesType & KsensorsMain::ReadConfig )
                        chip->readConfig();


                    if ( updatesType & KsensorsMain::UpdateIntervalList )
                        updateData.updateIntervalList.append ( chip->getSensorUpdateInterval( false ) );

                    if ( updatesType & KsensorsMain::GlobalTimer )
                    {
                        if ( chip->getMonitorized() )
                        {
// qDebug() << "\033[91m" << "qDebug(): ksensorsMain::updateSensorsStatus chip->getMonitorized()" << chip->getMonitorized() << " chip->getLongName() " << chip->getLongName() <<"\033[0m";

                            chip->globalTimerUpdate ( updateInterval, updateIntervalTimer );
                        }
                    }
                } // if ( sensorDevName.isEmpty() || ( chip->getLongName() ==  sensorDevName ) )


                QObjectList *list = ( QObjectList * )chip->getSensors();

                if( list )
                {
                    for ( QObjectList::ConstIterator itList = list->begin(); itList != list->end(); ++itList )
                    {
                        Sensor *sensor = ( Sensor * )( *itList );

                        if ( sensor )
                        {
                            if ( sensorName.isEmpty() || ( sensor->getLongName() == sensorName ) )
                            {
//                              setGroupConf ( sensor->objectName() );
                                setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );

                                qsSensorShowType = getEntryConf ( qsFL1( "show" ), qsFL1( "false" ), ksensorsSetting::AutoDefault );

                                sensorMeterTypes = MeterTypes::None;
                                if ( qsSensorShowType.indexOf ( qsFL1( "gauge" ) ) >= 0 )
                                    sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Gauge );
                                if ( qsSensorShowType.indexOf ( qsFL1( "chart" ) ) >= 0 )
                                    sensorMeterTypes = (MeterTypes)( sensorMeterTypes | MeterTypes::Chart );


                                if ( updatesType & KsensorsMain::ReadConfig )
                                {
                                    if ( gaugePanelsWindow )
                                        gaugePanelsWindow->updateWidget( sensor->getLongName(), updateData.withChange );
                                    else
                                    {
                                        if ( sensorMeterTypes & MeterTypes::Gauge )
                                            createShowWidget( updatesType );
                                    }

                                    if ( chartPanelsWindow )
                                        chartPanelsWindow->updateWidget( sensor->getLongName(), updateData.withChange );
                                    else
                                    {
                                        if ( sensorMeterTypes & MeterTypes::Chart )                                        
                                            createShowWidget( updatesType );
                                    }

                                    Q_EMIT sensor->panelDockConfigChanged();


                                    if ( sensor->getMonitorize() )
                                    {

                                    }
                                } // if ( updatesType & ksensorsMain::ReadConfig )

                            } // if ( sensorName.isEmpty() || ( sensor->getLongName() ==  sensorName ) )
                        }
                        else
                            std::abort();
                    } // for ( QObjectList::ConstIterator itList = list->begin(); itList != list->end(); ++itList )


                    if ( sensorDevName.isEmpty() || ( chip->getLongName() == sensorDevName ) )
                    {
                        if ( updatesType & KsensorsMain::ReadConfig )
                            chip->updateSensors( true );
                    }

                } // if( list )
            } // if( chip )
            else
                std::abort();;
        } // for ( QObjectList::ConstIterator it = chips->begin(); it != chips->end(); ++it )
    } // if( chips )


    if ( updatesType & KsensorsMain::Initialize &&  !( gaugePanelsWindow || chartPanelsWindow ) )    
        createShowWidget ( updatesType );
    
    if ( updatesType & KsensorsMain::ReadConfig && updateData.sensorName.isEmpty() )
    {
        if ( gaugePanelsWindow )
            gaugePanelsWindow->updateWidget( qsFL1( "" ), false );

        if ( chartPanelsWindow )
            chartPanelsWindow->updateWidget( qsFL1( "" ), false );
    }

} // void updateSensorsStatus ( Updates updatesType, struct T_UpdateData &updateData )

void KsensorsMain::createDockWidgets()
{
  const QObjectList *chips= sensorDevsList->getSensorsChips();

 countSensors= 0;
 if(chips)
 {
    for ( QObjectList::ConstIterator it = chips->begin(); it != chips->end(); ++it )       
    {
      SensorDev *chip = (SensorDev *)(*it);
       
      if ( chip )
      {
        
        QObjectList *list= (QObjectList *)chip->getSensors();
        if(list) 
        {
          for ( QObjectList::ConstIterator itList = list->begin(); itList != list->end(); ++itList )       
          {
            LMSensor *sensor = (LMSensor *)(*itList);
       
            if ( sensor )
            {
//              setGroupConf ( sensor->objectName() );
                setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
                if ( getEntryConf ( qsFL1( "showInDock" ), false, ksensorsSetting::AutoDefault ) )
                    createDockSensor(sensor);
            }
            else
              break;
          }
        }
        
       }
       else
         break;
       
    }    
 }
    
 if(countSensors==0) createDockSensor( nullptr );
}

void KsensorsMain::createAlarmControl()
{
 sensorsAlarm= new LMSensorsAlarms( sensorDevsList );
}


void KsensorsMain::minimizeRestoreWidget()
{
    Updates updatesType;
    
    updatesType = KsensorsMain::Default;
    
    if ( gaugePanelsWindow || chartPanelsWindow )
    {
        if ( gaugePanelsWindow )
        {
            delete gaugePanelsWindow;
            gaugePanelsWindow = nullptr;
        }

        if ( chartPanelsWindow )
        {
            delete chartPanelsWindow;
            chartPanelsWindow = nullptr;
        }
    }
    else               
        createShowWidget( updatesType );

    updateMenu ( gaugePanelsWindow, chartPanelsWindow );
}

void KsensorsMain::updateMenu ( bool withGauge, bool withChart )
{
  QString qsActionText;
  
  
 if(getPar ( qsFL1( "nodockopt" ), false ) && !( withGauge || withChart ) )
   close();
 else
 { 
    qsActionText = ( withGauge || withChart ) ? i18n( "&Close" ) : i18n( "&Restore" );
   
    qaMinimizeRestore->setText( qsActionText );
    
    const QObjectList *list= &children();
    
    if(list)
    {
        for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) 
        {
            LMSensorDockPanel *sensor = (LMSensorDockPanel *)(*it);
 
            if ( sensor )
            {
                sensor->updateMenu( qsActionText );
            }
            else
                break;      
        }
    }
 } // else (noDock && !sensorsWidget)
}

void KsensorsMain::createShowWidget ( Updates updatesType )
{
    int desktop;

    desktop = getEntryConf ( qsFL1( "desktop" ), -1, ksensorsSetting::AutoDefault );

    setGroupConf ( qsFL1( "General" ) );

// warning, noDock is tested
    if ( !( updatesType & KsensorsMain::Initialize ) || getEntryConf ( qsFL1( "showWidget" ), true, ksensorsSetting::AutoDefault ) )
    {

//    Updates updatesType;
    MeterType meterType;

    struct T_UpdateData updateData;

// warning, is need to check if any values ​​need to be set before calling the creation of the panel window and panels
/*    updatesType = ksensorsMain::Default | ksensorsMain::UpdateMeterSizeList;

    updateData.init();

    updateConfig ( updatesType, updateData ); */

    if ( !chartPanelsWindow )
    {
        meterType = MeterTypes::Chart;
        chartPanelsWindow = new PanelsWindow ( this, meterType );

        if ( chartPanelsWindow->getMeterTypes() & MeterTypes::Empty )
            delete chartPanelsWindow;
        else
        {
            connect( chartPanelsWindow, SIGNAL(rightMouseClicked(QMouseEvent *)), this, SLOT(mouseEventReceived(QMouseEvent *)) );
            connect( chartPanelsWindow, &PanelsWindow::destroyed, this, [=, this]() { this->updateMenu ( gaugePanelsWindow, false ); } );
        }
    }

    if ( !gaugePanelsWindow )
    {
        meterType = MeterTypes::Gauge;
        gaugePanelsWindow = new PanelsWindow ( this, meterType );

        if ( gaugePanelsWindow->getMeterTypes() & MeterTypes::Empty )
        {
            if ( chartPanelsWindow )
                delete gaugePanelsWindow;
            else
                gaugePanelsWindow->setMeterTypes ( meterType );
        }

        if ( gaugePanelsWindow )
        {
            connect( gaugePanelsWindow, SIGNAL(rightMouseClicked(QMouseEvent *)), this, SLOT(mouseEventReceived(QMouseEvent *)) );
            connect( gaugePanelsWindow, &PanelsWindow::destroyed, this, [=, this]() { this->updateMenu ( false, chartPanelsWindow ); } );
        }
    }

    if ( ( desktop != 0 ) && (desktop <= KX11Extras::numberOfDesktops() ) )
    {
        if ( gaugePanelsWindow )
            KX11Extras::setOnDesktop( gaugePanelsWindow->winId(), desktop );

        if ( chartPanelsWindow )
            KX11Extras::setOnDesktop( gaugePanelsWindow->winId(), desktop );
    }

    if ( gaugePanelsWindow )
        gaugePanelsWindow->show();

    if ( chartPanelsWindow )
        chartPanelsWindow->show();


//    TESTARE BENE
/*    updatesType = ksensorsMain::Default | ksensorsMain::UpdateMeterSizeList;

    updateData.init();

    updateConfig ( updatesType, updateData ); */

    } // if ( !( updatesType & ksensorsMain::Initialize ) || getEntryConf ( qsFL1( "showWidget" ), false, ksensorsSetting::AutoDefault ) )
}

void KsensorsMain::createConfigWidget()
{
 if(!sensorsCfg) {
   sensorsCfg= new KSensorsCfg( this );
 }
  
 sensorsCfg->show();
}

void KsensorsMain::close()
{
//  QCoreApplication::quit();
  
  this->deleteLater();
  
//  delete this;
}

void KsensorsMain::mouseEventReceived(QMouseEvent *e)
{
 if(e->type()==QEvent::MouseButtonRelease) {
   if( e->button()==Qt::LeftButton  ) minimizeRestoreWidget();
   if( e->button()==Qt::RightButton )
   {
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
       qmContext->popup(e->globalPos());
#else
       qmContext->popup(e->globalPosition().toPoint());
#endif
   }
 }
}

void KsensorsMain::createDockSensor(Sensor *sensor)
{
  if(countSensors==0) {
    const QObjectList *list= &children();
    
    if(list)
    {
      for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) 
      {
        LMSensorDockPanel *sensor = (LMSensorDockPanel *)(*it);
 
        if ( sensor )
        {
          delete sensor;        
        }
        else
          break;
        
      }
    }
  }
  const QString &name= sensor ?  sensor->getLongName() : QString();
  LMSensorDockPanel *dock= new LMSensorDockPanel( sensor, this, name );
  connect( dock, SIGNAL(mouseEvent(QMouseEvent *)), this, SLOT(mouseEventReceived(QMouseEvent *)) );
  
  dock->setParent( this );
//  dock->show();
  if ( !name.isEmpty() ) countSensors++;
}

void KsensorsMain::deleteDockSensor(LMSensorDockPanel *sensor)
{
 delete sensor;
 if(--countSensors==0) createDockSensor( nullptr );
}

void KsensorsMain::updateSensorsPanelsDocks( const QString &name, bool withChange )
{
    Updates updatesType;
    
    struct T_UpdateData updateData;

    if( !name.isEmpty() && name != qsFL1( "General" ) && name != qsFL1( "_init_" ) )
    {

 Sensor *sensor= sensorDevsList->getSensor(name);

//if((bool)sensor && sensor->getType()==LMSensor::lmTemp) {
//  setGroupConf ( name );
    if ( sensor )
        setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );       
    
    bool fShow= sensor && getEntryConf ( qsFL1( "showInDock" ), false, ksensorsSetting::AutoDefault );

   
   LMSensorDockPanel *dockSensor= findChild<LMSensorDockPanel *>( name );
   if( fShow!=(bool)dockSensor ) {
     if(dockSensor) deleteDockSensor(dockSensor);
     else           createDockSensor(sensor);
   } else {
     if(dockSensor!= nullptr )
       dockSensor->slotUpdateConfig();
   }
// }

    } // if( !name.isEmpty() && name != "General" && name != "_init_" ) 
//  else
    {
        //    TESTARE BENE
// warning, is need to check if any values ​​need to be set before calling the creation of the panel window and panels, or update interval
/*        updatesType = ksensorsMain::Default | ksensorsMain::UpdateMeterSizeList;

        updateData.init();

        updateConfig ( updatesType, updateData ); */


        updatesType = KsensorsMain::Default;
        if ( name == qsFL1( "_init_" ) )
            updatesType |= KsensorsMain::Initialize | KsensorsMain::UpdateSensorsList;
        updatesType |= KsensorsMain::ReadConfig | KsensorsMain::UpdateIntervalList;
/*        if ( name == qsFL1( "General" ) )
            updatesType |= ksensorsMain::UpdateMeterSizeList; */

        updateData.init();
        updateData.withChange = withChange;
        updateData.sensorName = name;

        updateConfig ( updatesType, updateData );

//    TESTARE BENE
        // in CFG se apply in tab sensors, imposta l'intervallo di default senza le righe qui sotto
        // se apply in tab preference (interval), imposta correttamente
        // l'update config iniziale forse non serve più con le righe qui sotto
        updatesType = KsensorsMain::Default | KsensorsMain::UpdateIntervalList;
    
        updateData.init();
        updateData.withChange = withChange;


        updateConfig ( updatesType, updateData );


/* warning, it would be better not to have the global timer active without any sensor enabled,
   since the sensor lists are monitored by default, the timer is activated) */
        if ( idTimer )
        {
            killTimer ( idTimer );
            idTimer = 0;

            if ( !idTimer )
                idTimer = startTimer( updateIntervalTimer );
        }

    } // else ( !name.isEmpty() )
}

void KsensorsMain::showAbout()
{  
  KAboutApplicationDialog *dialog;
  
  const KAboutData kadAboutData = KAboutData::applicationData();

  dialog= new KAboutApplicationDialog( kadAboutData );
  dialog->exec();
  delete dialog;
}

/***************************************************************************
    add meter size
 ***************************************************************************/

void KsensorsMain::setMeterSizeList()
{
    int iMaxCommDiv, iMin, iMax;

    QList<int> qlISize;
    
    gaugeIncrWidth = gaugeIncrHeight = chartIncrWidth = chartIncrHeight = 64;

    qlIGaugeSizeList.clear();

    qlIGaugeSizeList.append( 160 );
    qlIGaugeSizeList.append( 152 );
    qlIGaugeSizeList.append( 144 );
    qlIGaugeSizeList.append( 136 );
    qlIGaugeSizeList.append( 128 );
    qlIGaugeSizeList.append( 120 );
    qlIGaugeSizeList.append( 112 );
    qlIGaugeSizeList.append( 104 );
    qlIGaugeSizeList.append( 96 );
    qlIGaugeSizeList.append( 88 );
    qlIGaugeSizeList.append( 80 );
    qlIGaugeSizeList.append( 72 );
    qlIGaugeSizeList.append( 64 );
    qlIGaugeSizeList.append( 56 );
    qlIGaugeSizeList.append( 48 );

    qlIChartWidthList.append( 960 );
    qlIChartWidthList.append( 864 );
    qlIChartWidthList.append( 768 );
    qlIChartWidthList.append( 672 );
    qlIChartWidthList.append( 576 );
    qlIChartWidthList.append( 480 );
    qlIChartWidthList.append( 384 );
    qlIChartWidthList.append( 288 );
    qlIChartWidthList.append( 192 );
    qlIChartWidthList.append( 160 );
    qlIChartWidthList.append( 144 );
    qlIChartWidthList.append( 128 );
    qlIChartWidthList.append( 112 );
    qlIChartWidthList.append( 96 );

    qlIChartHeightList.append( 480 );
    qlIChartHeightList.append( 384 );
    qlIChartHeightList.append( 288 );
    qlIChartHeightList.append( 192 );
    qlIChartHeightList.append( 160 );
    qlIChartHeightList.append( 144 );
    qlIChartHeightList.append( 128 );
    qlIChartHeightList.append( 112 );
    qlIChartHeightList.append( 96 );

    qlISize = qlIGaugeSizeList;
    qlISize.append ( 64 );
    maxCommonDivisor ( qlISize, iMaxCommDiv, iMin, iMax );
    gaugeIncrWidth = gaugeIncrHeight = iMaxCommDiv;

    qlISize = qlIChartWidthList;
    qlISize.append ( 64 );
    maxCommonDivisor ( qlISize, iMaxCommDiv, iMin, iMax );
    chartIncrWidth = iMaxCommDiv;

    qlISize = qlIChartHeightList;
    qlISize.append ( 64 );
    maxCommonDivisor ( qlISize, iMaxCommDiv, iMin, iMax );
    chartIncrHeight = iMaxCommDiv;

} // void setMeterSizeList()

/***************************************************************************
    general timer event
 ***************************************************************************/

void KsensorsMain::timerEvent( QTimerEvent * )
{
    struct T_UpdateData updateData;

    updateSensorsConfig ( KsensorsMain::GlobalTimer, updateData );
}
